package net.confidentialfiles.OpenReference;

import java.util.*;
import java.sql.*;
import net.confidentialfiles.DBUtils.*;


public class Category extends DatabaseTable {

  static public String TABLE = "Category";

  static public String[] FIELDS = new String[] {
    "CategoryID",
    "Name",
    "OrderWeight",
    "ParentCategory"
  };

  static public String[] SEARCH_FIELDS = new String[] {
    "Name"
  };

  static public String[] USER_FIELDS = new String[] {
    "Name",
    "OrderWeight"
  };

  static public String PRIMARY_FIELD = "Name";

  static public final String ROOT_CATEGORY_ID="-1";
  static public final String ROOT_PARENT_CATEGORY="-2";

  public Category() throws Exception {
  }

  static public boolean isLeaf( String catid ) throws Exception {

    DatabaseConn dbc = new DatabaseConn();
    Connection conn = dbc.getConn();

    Vector v = getRows(conn, TABLE, FIELDS, 
                       "ParentCategory", catid);
    conn.close();
    if (v.size() == 0) {
      return true;
    } else {
      return false;
    }
  }

  static public synchronized long addCategory(Hashtable newCategory) throws Exception {

    DatabaseConn dbc = new DatabaseConn();
    Connection conn = dbc.getConn();

    long newID = getMaxID(conn, TABLE, "CategoryID") + 1;
    newCategory.put("CategoryID", (new Long(newID)).toString() );
    insertRow(conn, TABLE, FIELDS, newCategory);
    conn.close();
    return newID;
  }

  static public int deleteCategory(String catid) throws Exception {
    DatabaseConn dbc = new DatabaseConn();
    Connection conn = dbc.getConn();
    int res = deleteRows(conn, TABLE, "CategoryID", catid);
    conn.close();
    return res;
  }

  static public boolean deleteCatNode(String catid) throws Exception {
    DatabaseConn dbc = new DatabaseConn();
    Connection conn = dbc.getConn();

    Hashtable p = getParent(catid), h;
    Vector v = getSubCategories(catid);
    for (int i=0; i<v.size(); i++) {
      h = (Hashtable) v.elementAt(i);
      h.put("ParentCategory",
            (String) p.get("CategoryID") );
      updateCategory( h );
    }
    int n = deleteRows(conn, TABLE, "CategoryID", catid);
    conn.close();
    if ( n == 1 ) {
      return true;
    } else {
      return false;
    }
  }

  static public synchronized Hashtable getParent( String catid) throws Exception {

    DatabaseConn dbc = new DatabaseConn();
    Connection conn = dbc.getConn();

    String SQLstr = "SELECT ParentCategory FROM Category " +
                    "WHERE CategoryID = \"" + catid + "\" ";
    Statement stmt = conn.createStatement();
    ResultSet rs = stmt.executeQuery( SQLstr );
    rs.next();
    String parentcatid = rs.getString("ParentCategory");
    rs.close();
    stmt.close();

    if ( ROOT_PARENT_CATEGORY.equals(parentcatid) ) {
      conn.close();
      return null;
    } else {
      Vector v = getRows(conn, TABLE, FIELDS, "CategoryID", parentcatid);
      conn.close();
      return (Hashtable) v.elementAt(0);
    }
  }

  static public boolean swapID(String id1, String id2) throws Exception {

    //id1 and id2 MUST be father and son to begin with.
    //This function basically swaps the father and son positions

    DatabaseConn dbc = new DatabaseConn();
    Connection conn = dbc.getConn();

    String SQLstr = "UPDATE " + TABLE + " SET CategoryID" +
                    " = \'" + id2 + "\' " +
                    "WHERE CategoryID = \'" +
                    id1 + "\' ";
    Statement stmt = conn.createStatement();
    int n1 = stmt.executeUpdate( SQLstr );

    SQLstr = "UPDATE " + TABLE + " SET CategoryID" +
             " = \'" + id1 + "\', ParentCategory" +
             " = \'" + id2 + "\' " +
             "WHERE CategoryID = \'" +
             id2 + "\' AND ParentCategory" + 
             " = \'" +
             id1 + "\' "; 
    int n2 = stmt.executeUpdate( SQLstr );

    stmt.close();
    conn.close();

    if( n1 == 1 && n2 == 1 ) {
      return true;
    } else {
      return false;
    }
  }

  static public synchronized Vector getSiblings(String catid) throws Exception {

    DatabaseConn dbc = new DatabaseConn();
    Connection conn = dbc.getConn();

    Vector result = new Vector();
    String SQLstr = "SELECT ParentCategory FROM CategoryTable " +
                    "WHERE CategoryID = \"" + catid + "\" ";
    Statement stmt = conn.createStatement();
    ResultSet rs = stmt.executeQuery( SQLstr );
    rs.next();
    String parentcatid = rs.getString("ParentCategory");
    rs.close();
    stmt.close();

    result = getRows(conn, TABLE, FIELDS, "ParentCategory", parentcatid);
    conn.close();
    return result;
  }

  static public boolean updateCategory(Hashtable newValues) throws Exception {

    DatabaseConn dbc = new DatabaseConn();
    Connection conn = dbc.getConn();

    int n = updateRows(conn, TABLE, FIELDS, "CategoryID", (String) newValues.get("CategoryID"), newValues);
    conn.close();
    return n==1 ? true : false;
  }

  static public Vector getSubCategories(String catid) throws Exception {

    DatabaseConn dbc = new DatabaseConn();
    Connection conn = dbc.getConn();

    Vector result = getRows(conn, TABLE, FIELDS, 
                            "ParentCategory", catid);
    conn.close();
    return result;
  }

  static public Vector getSubCategories(String catid, String order) throws Exception {

    DatabaseConn dbc = new DatabaseConn();
    Connection conn = dbc.getConn();

    Vector result = getRows(conn, TABLE, FIELDS, 
                            "ParentCategory", 
                            catid, order, "ASC");
    conn.close();
    return result;
  }

  static public Hashtable getCatInfo(String catid) throws Exception {

    if ( ROOT_PARENT_CATEGORY.equals(catid) ) {
      return null;
    }

    DatabaseConn dbc = new DatabaseConn();
    Connection conn = dbc.getConn();

    Vector v = getRows(conn, TABLE, FIELDS, 
                       "CategoryID", catid);
    conn.close();
    return (Hashtable) v.elementAt(0);
  }

  static public String primaryField() throws Exception {
    return PRIMARY_FIELD;
  }

  public Vector userFields() throws Exception {
    Vector v = new Vector();
    for(int i=0; i<FIELDS.length; i++) {
      if ( !"CategoryID".equals(FIELDS[i]) && 
           !"ParentCategory".equals(FIELDS[i]) ) {
        v.add( FIELDS[i] );
      }
    }
    return v;
  }

  static public Vector tracePath(String catid) throws Exception {

    DatabaseConn dbc = new DatabaseConn();
    Connection conn = dbc.getConn();

    Vector result = new Vector();
    String parentcatid = "", current = catid;

    while ( !ROOT_PARENT_CATEGORY.equals(current) ) {
      Vector v = getRows(conn, TABLE, FIELDS, 
                         "CategoryID", current);
      if ( v == null ) throw new Exception("Not a valid CategoryID");
      if ( v.size() != 1 ) throw new Exception("Not a valid CategoryID too");
      Hashtable h = (Hashtable) v.elementAt(0);
      parentcatid = (String) h.get("ParentCategory");
      current = parentcatid;
      result.add( h );
    }
    conn.close();
    return result;
  }

  static public Vector search(String[] keywords, String searchOp) throws Exception {

    DatabaseConn dbc = new DatabaseConn();
    Connection conn = dbc.getConn();

    Vector res =  DatabaseTable.search(conn, TABLE, "CategoryID",
                        SEARCH_FIELDS, keywords, searchOp);
    conn.close();
    return res;
  }


}
