package net.confidentialfiles.OpenReference;

import java.sql.*;
import java.util.*;
import net.confidentialfiles.DBUtils.*;

public class Reference extends DatabaseTable {

  static public String TABLE = "Reference";

  static public String[] FIELDS = new String[] {
    "ReferenceID",
    "UserID",
    "PostDate",
    "URL",
    "Name",
    "Author",
    "Memo",
    "BibCode",
    "View",
    "UserList"
  };

  static public String[] SEARCH_FIELDS = new String[] {
    "Name",
    "Author",
    "Memo"
  };


  public Reference() throws Exception {
  }

  static public synchronized long addReference(Hashtable newReference) throws Exception {

    DatabaseConn dbc = new DatabaseConn();
    Connection conn = dbc.getConn();

    long newID = getMaxID(conn, TABLE, "ReferenceID") + 1;
    newReference.put("ReferenceID", (new Long(newID)).toString() );
    newReference.put("PostDate", getCurrentTime(conn) );

    insertRow(conn, TABLE, FIELDS, newReference);
    conn.close();
    return newID;
  }

  static public boolean updateReference(Hashtable newValues) throws Exception {

    DatabaseConn dbc = new DatabaseConn();
    Connection conn = dbc.getConn();

    int n = updateRows(conn, TABLE, FIELDS, "ReferenceID", (String)newValues.get("ReferenceID"), newValues);
    conn.close();
    return n==1 ? true : false;
  }

  static public Hashtable getReference(String ID) throws Exception {

    DatabaseConn dbc = new DatabaseConn();
    Connection conn = dbc.getConn();

    Vector v = getRows(conn, TABLE, FIELDS, "ReferenceID", ID);
    conn.close();
    return (Hashtable) v.elementAt(0);
  }

  static public Vector getAllReferences() throws Exception {

    DatabaseConn dbc = new DatabaseConn();
    Connection conn = dbc.getConn();

    Vector res = getAllRows(conn, TABLE, FIELDS, 
                      "PostDate", "ASC");
    return res;
  }

  static public Vector getRecentReferences() throws Exception {
    try {
//      java.util.Date d = new java.util.Date();
//      int y = d.getYear() + 1900;
//      String limitTime = y + "-" + d.getMonth() + "-" + d.getDate() + " " +
//                     "00:00:00";
//      System.out.println("Time is " + limitTime);

      Calendar today = new GregorianCalendar();
      String limitTime = today.get(Calendar.YEAR) + "-" + 
                         today.get(Calendar.MONTH)  + "-" + 
                         today.get(Calendar.DAY_OF_MONTH) + " " + "00:00:00";
//      System.out.println("Time is " + limitTime);

      DatabaseConn dbc = new DatabaseConn();
      Connection conn = dbc.getConn();

      String SQLstr = "SELECT * FROM " + TABLE + " WHERE PostDate > \'" + 
                      limitTime + "\' ORDER BY PostDate ASC";
      Statement stmt = conn.createStatement();
      ResultSet rs = stmt.executeQuery(SQLstr);

      Vector result = new Vector();
      while (rs.next()) {
        Hashtable entry = new Hashtable();
        for(int i=0; i<FIELDS.length; i++) {
          if ( rs.getString(FIELDS[i]) != null ) {
            entry.put(FIELDS[i], rs.getString(FIELDS[i]));
          }
        }
        result.add( entry );
      }
      rs.close();
      stmt.close();
      conn.close();
      return result;
    } catch (Exception e) {
      throw new Exception("Error in Entity getEntries" + e.getMessage() );
    }
  }

/*********************************************************
  static public Vector getViewableReferences(String cName, String cID) throws Exception {
    try {
      DatabaseConn dbc = new DatabaseConn();
      Connection conn = dbc.getConn();

      String SQLstr = "SELECT * FROM " + TABLE + 
             " WHERE (View = \'AllUsers\') OR " + 
             " ( (View = \'Myself\') AND (UserID = " + cID + ") ) OR " +
             " ( (View = \'UserList\') AND (UserList LIKE \'%" + cName + "%\') )";
      Statement stmt = conn.createStatement();
      ResultSet rs = stmt.executeQuery(SQLstr);

      Vector result = new Vector();
      while (rs.next()) {
        SQLstr = "SELECT * FROM " + User.TABLE + " WHERE (UserID = \'" +
                 rs.getString("UserID") + "\') AND " +
                 "( (View = \'AllUsers\') OR " +
                 " ( (View = \'Myself\') AND (UserID = " + cID + ") ) OR " +
                 " ( (View = \'UserList\') AND (UserList LIKE \'%" + cName + "%\') )";
        ResultSet tmprs = stmt.executeQuery(SQLstr);
        if (tmprs.next()) {
          Hashtable entry = new Hashtable();
          for(int i=0; i<FIELDS.length; i++)
            if ( rs.getString(FIELDS[i]) != null ) {
              entry.put(FIELDS[i], rs.getString(FIELDS[i]));
            }
          result.add( entry );
        }
        tmprs.close();
      }
      rs.close();
      stmt.close();
      conn.close();
      return result;
    } catch (Exception e) {
      throw new Exception("Error in Entity getEntries" + e.getMessage() );
    }
  }
**********************************************************/

  static public int deleteReference(String ID) throws Exception {

    DatabaseConn dbc = new DatabaseConn();
    Connection conn = dbc.getConn();

    int res = deleteRows(conn, TABLE, "ReferenceID", ID);
    conn.close();
    return res;
  }

  static public Hashtable getUser(String ID) throws Exception {

    DatabaseConn dbc = new DatabaseConn();
    Connection conn = dbc.getConn();

    Vector v =  getEntries(conn, User.TABLE, User.FIELDS,
                           TABLE, "UserID", "ReferenceID", ID);
    conn.close();
    return (Hashtable) v.elementAt(0);
  }

  static public Vector getReferencesByUser(String UserID) throws Exception {

    DatabaseConn dbc = new DatabaseConn();
    Connection conn = dbc.getConn();

    Vector res =  getRows(conn, TABLE, FIELDS,
                    "UserID", UserID);
    conn.close();
    return res;
  }

  static public Vector search(String query) throws Exception {

    DatabaseConn dbc = new DatabaseConn();
    Connection conn = dbc.getConn();

    Vector res =  search(conn, TABLE, FIELDS, SEARCH_FIELDS, query);

    conn.close();
    return res;
  }

}

