package net.confidentialfiles.DBUtils;

import java.sql.*;
import java.util.*;

public class DatabaseTable {

  public DatabaseTable() {
  }

  static protected synchronized long getMaxID(Connection conn, String tableName, String idFieldName) throws Exception {

    // 
    // Return the maximum value in field `idFieldName' of table 
    // `tableName'.
    // 

    try {                         

      String SQLstr = "SELECT MAX(" + idFieldName + ") FROM " + tableName;
      Statement stmt = conn.createStatement();
      ResultSet rs = stmt.executeQuery(SQLstr);
      rs.next();
      long l = rs.getLong(1);
      rs.close();
      stmt.close();
      return l;
    } catch (Exception e) {
      return 0;
    }
  }

  static protected synchronized int insertRow(Connection conn, String tableName, String[] fieldNames, Hashtable entry) throws Exception {

    //
    // Insert or update a row `entry' in table `tableName' with
    // fields `fieldNames'
    //                        

    try {

      // get the field name list and the value list
      String nameList = DatabaseUtil.getNameList(fieldNames, entry);
      String valueList = DatabaseUtil.getValueList(fieldNames, entry);
      
      // insert the given entry to the given table
      String SQLstr = "INSERT INTO " + tableName + " " + nameList + 
                      " values " + valueList;
      Statement stmt = conn.createStatement();
      int n = stmt.executeUpdate(SQLstr);
      stmt.close();
      return n;
    } catch (Exception e) {
      throw new Exception("Error in Entity insertEntry" + e.getMessage() );
    }
  }

  static protected synchronized int getRowCount(Connection conn, String tableName) throws Exception {
    try {
      int result = 0;

      // get an entry from `tableName' whose `fieldName' equals `match'
      String SQLstr = "SELECT COUNT(*) FROM " + tableName; 
      Statement stmt = conn.createStatement();
      ResultSet rs = stmt.executeQuery(SQLstr);
      if (rs.next()) {
        result = rs.getInt(1);
      }
      rs.close();
      stmt.close();
      return result;
    } catch (Exception e) {
      throw new Exception("Error in getCounts" + e.getMessage() );
    }
  }

  static protected synchronized Vector getRows(Connection conn, String tableName, String[] fieldNames, String matchName, String matchValue) throws Exception {

    //
    // Get a row in table `tableName' with fields `fieldNames'
    // where `matchName' field has exact `matchValue', and return it
    // as a hashtable.
    //                        

    try {

      // get an entry from `tableName' whose `fieldName' equals `match'
      String SQLstr = "SELECT " + DatabaseUtil.getNameList(fieldNames) +
                      " FROM " + tableName + " WHERE " + 
                      matchName + " = \'" + matchValue + "\' ";
      Statement stmt = conn.createStatement();
      ResultSet rs = stmt.executeQuery(SQLstr);
      
      Vector result = new Vector();
      while (rs.next()) {
        Hashtable entry = new Hashtable();
        for(int i=0; i<fieldNames.length; i++)
          if ( rs.getString(fieldNames[i]) != null ) {
            entry.put(fieldNames[i], rs.getString(fieldNames[i]));
          }
        result.add( entry );
      }
      rs.close();
      stmt.close();
      return result;
    } catch (Exception e) {
      throw new Exception("Error in Entity getEntries" + e.getMessage() );
    }
  }

  static protected synchronized Vector getRows(Connection conn, String tableName, String[] fieldNames, String matchName, String matchValue, String orderName, String orderOp) throws Exception {

    try {

      // get an entry from `tableName' whose `fieldName' equals `match'
      String SQLstr = "SELECT " + DatabaseUtil.getNameList(fieldNames) +
                      " FROM " + tableName + " WHERE " + 
                      matchName + " = \'" + matchValue + "\' " + "ORDER BY " +
                      orderName + " " + orderOp;
      Statement stmt = conn.createStatement();
      ResultSet rs = stmt.executeQuery(SQLstr);
      
      Vector result = new Vector();
      while (rs.next()) {
        Hashtable entry = new Hashtable();
        for(int i=0; i<fieldNames.length; i++)
          if ( rs.getString(fieldNames[i]) != null ) {
            entry.put(fieldNames[i], rs.getString(fieldNames[i]));
          }
        result.add( entry );
      }
      rs.close();
      stmt.close();
      return result;
    } catch (Exception e) {
      throw new Exception("Error in Entity getEntries" + e.getMessage() );
    }
  }


  static protected synchronized Vector getRows(Connection conn, String tableName, String[] fieldNames, String[] matchNames, String matchValues[], String matchOp) throws Exception {

    //
    // Get a row in table `tableName' with fields `fieldNames'
    // where `matchName[]' fields have `matchValues[]' with relations, 
    // matchOp (AND or OR), and return it
    // as a hashtable.
    //                        

    try {

      String cond = DatabaseUtil.getMatchCondition(matchNames, matchValues, matchOp); 
      String SQLstr = "SELECT " + DatabaseUtil.getNameList(fieldNames) +
                      " FROM " + tableName + " WHERE " + cond ;
      Statement stmt = conn.createStatement();
      ResultSet rs = stmt.executeQuery(SQLstr);
      
      Vector result = new Vector();
      while (rs.next()) {
        Hashtable entry = new Hashtable();
        for(int i=0; i<fieldNames.length; i++)
          if ( rs.getString(fieldNames[i]) != null ) {
            entry.put(fieldNames[i], rs.getString(fieldNames[i]));
          }
        result.add( entry );
      }
      rs.close();
      stmt.close();
      return result;
    } catch (Exception e) {
      throw new Exception("Error in Entity getEntries" + e.getMessage() );
    }
  }

  static protected synchronized Vector getRows(Connection conn, String tableName, String[] fieldNames, String[] matchNames, String matchValues[], String matchOp, String orderName, String orderOp) throws Exception {

    try {

      String cond = DatabaseUtil.getMatchCondition(matchNames, matchValues, matchOp); 
      String SQLstr = "SELECT " + DatabaseUtil.getNameList(fieldNames) +
                      " FROM " + tableName + " WHERE " + cond + 
                      " ORDER BY " + orderName + " " + orderOp;
      Statement stmt = conn.createStatement();
      ResultSet rs = stmt.executeQuery(SQLstr);
      
      Vector result = new Vector();
      while (rs.next()) {
        Hashtable entry = new Hashtable();
        for(int i=0; i<fieldNames.length; i++)
          if ( rs.getString(fieldNames[i]) != null ) {
            entry.put(fieldNames[i], rs.getString(fieldNames[i]));
          }
        result.add( entry );
      }
      rs.close();
      stmt.close();
      return result;
    } catch (Exception e) {
      throw new Exception("Error in Entity getEntries" + e.getMessage() );
    }
  }


  static protected synchronized int deleteRows(Connection conn, String tableName, String matchName, String matchValue) throws Exception {

    try {

      String SQLstr = "DELETE FROM " + tableName + " WHERE " + 
                      matchName + " = \'" + matchValue + "\' ";
      Statement stmt = conn.createStatement();
      int n = stmt.executeUpdate(SQLstr);
      stmt.close();
      return n;
    } catch (Exception e) {
      throw new Exception("Error in Entity deleteEntries" + e.getMessage() );
    }
  }

  static protected synchronized int deleteRows(Connection conn, String tableName, String[] matchNames, String matchValues[], String matchOp) throws Exception {

    try {

      String cond = DatabaseUtil.getMatchCondition(matchNames, matchValues, matchOp); 
      String SQLstr = "DELETE FROM " + tableName + " WHERE " + cond ;
      Statement stmt = conn.createStatement();
      int n = stmt.executeUpdate(SQLstr);
      stmt.close();
      return n;
    } catch (Exception e) {
      throw new Exception("Error in Entity deleteEntries" + e.getMessage() );
    }
  }


  static protected synchronized int updateRows(Connection conn, String tableName, String[] fieldNames, String matchName, String matchValue, Hashtable newEntry) throws Exception {

    try {

      // get an entry from `tableName' whose `fieldName' equals `match'
      String pairList = DatabaseUtil.getPairList(fieldNames, newEntry);
      String SQLstr = "UPDATE " + tableName + " SET " + pairList +
                      " WHERE " + matchName + " = \"" + matchValue + "\" ";
      Statement stmt = conn.createStatement();
      int n = stmt.executeUpdate(SQLstr);
      stmt.close();
      return n;
    } catch (Exception e) {
      throw new Exception("Error in Entity updateEntries" + e.getMessage() );
    }
  }

  static protected synchronized int updateRows(Connection conn, String tableName, String[] fieldNames, String[] matchNames, String[] matchValues, String matchOp, Hashtable newEntry) throws Exception {

    try {

      String cond = DatabaseUtil.getMatchCondition(matchNames, matchValues, matchOp); 
      String pairList = DatabaseUtil.getPairList(fieldNames, newEntry);
      String SQLstr = "UPDATE " + tableName + " SET " + pairList + cond ;
      Statement stmt = conn.createStatement();
      int n = stmt.executeUpdate(SQLstr);
      stmt.close();
      return n;
    } catch (Exception e) {
      throw new Exception("Error in Entity updateEntries" + e.getMessage() );
    }
  }

  static protected synchronized Vector getAllRows(Connection conn, String tableName, String[] fieldNames) throws Exception {
    //
    // Get all rows in table `tableName' with fields `fieldNames' and
    // return it as a vector.
    //                        

    try {

      String SQLstr = "SELECT " + DatabaseUtil.getNameList(fieldNames) +
                      " FROM " + tableName;
      Statement stmt = conn.createStatement();
      ResultSet rs = stmt.executeQuery(SQLstr);
      
      Vector result = new Vector();
      while (rs.next()) {
        // store the query result to the hashtable `entry'
        Hashtable entry = new Hashtable();
        for(int i=0; i<fieldNames.length; i++)
          if ( rs.getString(fieldNames[i]) != null ) {
            entry.put(fieldNames[i], rs.getString(fieldNames[i]));
          }
        result.add(entry);
      }
      rs.close();
      stmt.close();
      return result;
    } catch (Exception e) {
      throw new Exception("Error in Entity getAllEntries" + e.getMessage() );
    }
  }

  static protected synchronized Vector getAllRows(Connection conn, String tableName, String[] fieldNames, String orderName, String orderOp) throws Exception {
    try {

      String SQLstr = "SELECT " + DatabaseUtil.getNameList(fieldNames) +
                      " FROM " + tableName + 
                      " ORDER BY " + orderName + " " + orderOp;
      Statement stmt = conn.createStatement();
      ResultSet rs = stmt.executeQuery(SQLstr);
      
      Vector result = new Vector();
      while (rs.next()) {
        // store the query result to the hashtable `entry'
        Hashtable entry = new Hashtable();
        for(int i=0; i<fieldNames.length; i++)
          if ( rs.getString(fieldNames[i]) != null ) {
            entry.put(fieldNames[i], rs.getString(fieldNames[i]));
          }
        result.add(entry);
      }
      rs.close();
      stmt.close();
      return result;
    } catch (Exception e) {
      throw new Exception("Error in Entity getAllEntries" + e.getMessage() );
    }
  }
  
  static protected synchronized String getCurrentTime(Connection conn) throws Exception {
    //
    // Return the current time.
    //

    try {

      String SQLstr = "SELECT now()";
      Statement stmt = conn.createStatement();
      ResultSet rs = stmt.executeQuery(SQLstr);
      rs.next();
      String curTime = rs.getString(1);
      rs.close();
      stmt.close();
      return curTime;
    } catch (Exception e) {
      throw new Exception("Error in Entity gettime" + e.getMessage() );
    }
  }


  static public synchronized Vector getEntries(Connection conn, String entityTableName, String[] entityTableFields, String relationTableName, String idFieldName, String matchName, String matchValue) throws Exception {

    //
    // Return all the rows in `entityTableName' where
    // the values of `idFieldName' in both `entityTableName' and 
    // `relationTableName' are the same, and `matchName' contains
    // `matchValue' in `relationTableName'.
    //

    try {

      String SQLstr = "SELECT " + entityTableName + ".* FROM " + 
                      entityTableName + " , " + relationTableName +
                      " WHERE " + entityTableName + "." + idFieldName + 
                      " = " + relationTableName + "." + idFieldName + 
                      " AND " + relationTableName + "." + matchName + 
                      " = \'" + matchValue + "\'";
      Statement stmt = conn.createStatement();
      ResultSet rs = stmt.executeQuery( SQLstr );

      Vector result = new Vector();
      while (rs.next()) {
        Hashtable h = new Hashtable();
        for(int i=0; i<entityTableFields.length; i++) {
          String tmp = rs.getString(entityTableName+"."+entityTableFields[i]);
          if ( tmp != null ) {
            h.put(entityTableFields[i], tmp);
          }
        }
        result.add( h );
      }
      rs.close();
      stmt.close();
      
      return result;
    } catch (Exception e) {
      throw new Exception("Error in Relation getEntries" + e.getMessage() );
    }
  }

  static public synchronized Vector getEntries(Connection conn, String entityTableName, String[] entityTableFields, String relationTableName, String idFieldName, String[] matchName, String[] matchValue, String matchOp) throws Exception {

    //Boy, this tought. I will not implement it until absolutely necessary
    throw new Exception("not implemented");

  }

  static public synchronized Vector search(Connection conn, String tableName, String[] returnFields, String[] searchFields, String query) throws Exception {

    try {

      StringTokenizer st = new StringTokenizer(query);
      String [] keywords = new String[ st.countTokens() ];
      int k = 0;
      while (st.hasMoreTokens()) {
        keywords[k] = st.nextToken();
        k++;
      }

      String  SQLstr = " SELECT * FROM " + tableName + " WHERE ";
      for (int i=0; i<keywords.length-1; i++) {
        SQLstr = SQLstr + " ( ";
        for (int j=0; j<searchFields.length-1; j++) {
          SQLstr = SQLstr + " " + searchFields[j] + " LIKE \'%" + keywords[i] 
                          + "%\' OR ";
        }
        SQLstr = SQLstr + " " + searchFields[searchFields.length-1] 
                        + " LIKE \'%" + keywords[i] + "%\' ) AND ";
      }
      SQLstr = SQLstr + " ( ";
      for (int j=0; j<searchFields.length-1; j++) {
        SQLstr = SQLstr + " " + searchFields[j] + " LIKE \'%" 
                        + keywords[keywords.length-1] + "%\' OR ";
      }
      SQLstr = SQLstr + " " + searchFields[searchFields.length-1]
                      + " LIKE \'%" + keywords[keywords.length-1] + "%\' ) ";


      Statement stmt = conn.createStatement();
      ResultSet rs = stmt.executeQuery(SQLstr);

      Vector result = new Vector();
      while (rs.next()) {
        Hashtable entry = new Hashtable();
        for( int i=0; i<returnFields.length; i++)
          if ( rs.getString(returnFields[i]) != null ) {
            entry.put(returnFields[i], rs.getString(returnFields[i]));
          }
        result.add( entry );
      }
      rs.close();
      stmt.close();
      return result;
    } catch (Exception e) {
      throw new Exception("Error in Entity search" + e.getMessage() );
    }
  }


  static public synchronized Vector search(Connection conn, String tableName, String idFieldName, String[] searchFields, String[] keywords, String searchOp) throws Exception {
    try {

      Vector result = new Vector();
      String cond = "";

      for (int i=0; i<searchFields.length-1; i++) {
        for (int j=0; j<keywords.length-1; j++) {
          cond += searchFields[i] + " LIKE \'%" + keywords[j] + "%\' OR ";
        }
        cond += searchFields[i] + " LIKE \'%" +
                keywords[keywords.length-1] + "%\' " + searchOp + " ";
      }

      for (int j=0; j<keywords.length-1; j++) {
        cond += searchFields[searchFields.length-1] + " LIKE \'%" +
                keywords[j] + "%\' OR ";
      }
      cond += searchFields[searchFields.length-1] + " LIKE \'% " +
              keywords[keywords.length-1] + "%\' ";

      String SQLstr = "SELECT " + idFieldName + " FROM " + tableName +
                      " WHERE " + cond;
      Statement stmt = conn.createStatement();
      ResultSet rs = stmt.executeQuery(SQLstr);
      while ( rs.next() ) {
        result.add( rs.getString(1) );
      }
      rs.close();
      stmt.close();
      return result;

    } catch (Exception e) {
      throw new Exception("Error in Entity search" + e.getMessage() );
    }
  }

  static public synchronized boolean isNameAvailable(Connection conn, String tableName, String fieldName, String fieldValue) throws Exception {
    //
    // Return true if value `fieldValue' does not appear in field `fieldName'
    // of table `tableName'.
    //

    try {

      if ( fieldValue == null ) return false;

      String SQLstr = "SELECT " + fieldName + " FROM " + tableName +
                      " WHERE " + fieldName + " = \'" + fieldValue + "\'";
      Statement stmt = conn.createStatement();
      ResultSet rs = stmt.executeQuery(SQLstr);
      boolean result = rs.next() ? false : true;
      rs.close();
      stmt.close();
      return result;
    } catch (Exception e) {
      throw new Exception("Error in Entity isNameAvailable" + e.getMessage() );
    }
  }

}
