package net.confidentialfiles.DBUtils;

import java.util.*;

public class DatabaseUtil {

  public static String getNameList(String[] names, Hashtable values) {

    //
    // Return a list of names in the following format from array `names':
    // (<name1>,<name2>,...,<nameN>) Only the names also appear in
    // values table get returned.
    //
    StringBuffer nameList = new StringBuffer();
    int i;
    nameList.append("(");
    // get the first specified field
    for(i=0; i<names.length; i++) {
      if (values.get(names[i])!=null) {
        nameList.append( names[i] );
        break;
      }
    }
    // get subsequent specified field
    for(i=i+1; i<names.length; i++) {
      if (values.get(names[i])!=null) {
        nameList.append(" , ");
        nameList.append(names[i]);
      }
    }
    nameList.append(")");
    return nameList.toString();
  }

  public static String getNameList(String[] names) {
    String nameList ="";
    for (int i=0; i<names.length-1; i++) {
      nameList = nameList + names[i] + ", ";
    }
    nameList = nameList + names[names.length-1];
    return nameList;
  }

  public static String getValueList(String[] names, Hashtable values)
       throws Exception {

    //
    // Return a list of values in the following format based on `names' and
    // values stored in `entry':
    //
    // ('<value1>','<value2>',...,'<valueN>') Only non Null values are
    // returned. This method must be used with the above getNameList()
    // to get a correct namelist
    // We do NOT use "" since not all fields are Strings
    //
    StringBuffer valueList = new StringBuffer();
    int i;
    valueList.append("(");
    // get the first specified field
    for(i=0; i<names.length; i++) {
      if (values.get(names[i])!=null) {
        if ( !isNullObj( values.get(names[i]) ) ) {
          valueList.append("\'" + values.get(names[i]) + "\'");
          break;
        } else {
          valueList.append( "NULL" );
          break;
        }
      }
    }
    // get subsequent specified field
    for(i=i+1; i<names.length; i++) {
      if (values.get(names[i])!=null) {
        if ( !isNullObj( values.get(names[i]) ) ) {
          valueList.append(" , ");
          valueList.append("\'" + values.get(names[i]) + "\'");
        } else {
          valueList.append(" , ");
          valueList.append( "NULL" );
        }
      }
    }
    valueList.append(")");
    return valueList.toString();
  }

  public static String getPairList(String[] names, Hashtable values) throws Exception {

    //
    // Return a list of names in the following format based on `names' and
    // values stored in `entry':
    //
    // <name1>='<value1>',<name2>='<value2>',...,<nameN>='<valueN>')
    // only non Null pairs are returned
    // We do NOT use "" since not all fields are Strings
    //

    StringBuffer pairList = new StringBuffer();
    int i;
    // get the first specified field
    for(i=0; i<names.length; i++) {
      if (values.get(names[i])!=null) {
        if ( !isNullObj( values.get(names[i]) ) ) {
          pairList.append(names[i]);
          pairList.append("=");
          pairList.append("\'" + (String)values.get(names[i]) + "\'");
          break;
        } else {
          pairList.append(names[i]);
          pairList.append("=");
          pairList.append( "NULL" );
          break;
        }
      }
    }
    // get subsequent specified field
    for(i=i+1; i<names.length; i++) {
      if (values.get(names[i])!=null) {
        if ( !isNullObj( values.get(names[i]) ) ) {
          pairList.append(" , ");
          pairList.append(names[i]);
          pairList.append("=");
          pairList.append("\'" + (String)values.get(names[i]) + "\'");
        } else {
          pairList.append(" , ");
          pairList.append(names[i]);
          pairList.append("=");
          pairList.append( "NULL" );
        }
      }
    }
    return pairList.toString();
  }


  public static String getMatchCondition(String[] fieldNames, String[] fieldValues, String matchOp) {
    //
    // Return a SQL condition clause based on the field name array
    // `fieldNames' and the field value array `fieldValues'.
    //

    StringBuffer condition = new StringBuffer();
    for(int i=0; i<fieldNames.length-1; i++) {
      condition.append(fieldNames[i] + "=\'" + fieldValues[i] + "\'");
      condition.append(" " + matchOp + " ");
    }
    condition.append(fieldNames[fieldNames.length-1] + "=\'" + fieldValues[fieldNames.length-1] + "\'");
    return condition.toString();
  }

  public static String escapeString(String str) {
    return escapeString(str, "plain");
  }
  public static String escapeString(String str, String mode) {

    StringBuffer result = new StringBuffer();

    if ( mode.equals("html") ) { 
      for (int i=0; i < str.length(); i++) {
        if ( str.charAt(i) == 0 ) {
          result.append( "\\0" );
        } else if ( str.charAt(i) == 92 ) {
          result.append( "\\\\" );
        } else if ( str.charAt(i) == 39 ) {
          result.append( "\\\'" );
        } else if ( str.charAt(i) == 34 ) {
          result.append( "\\\"" );
        } else {
          result.append( str.charAt(i) );
        }
      }
    }
    if ( mode.equals("plain") ) { 
      for (int i=0; i < str.length(); i++) {
        if ( str.charAt(i) == 0 ) {
          result.append( "\\0" );
        } else if ( str.charAt(i) == 92 ) {
          result.append( "\\\\" );
        } else if ( str.charAt(i) == 39 ) {
          result.append( "\\\'" );
        } else if ( str.charAt(i) == 34 ) {
          result.append( "\\\"" );
        } else if ( str.charAt(i) == 60 ) {
          result.append( "&lt;" );
        } else {
          result.append( str.charAt(i) );
        }
      }
    }

    return result.toString();
  }

  public static boolean isNullObj(Object o) throws Exception {
    return Class.forName("net.confidentialfiles.DBUtils.NullValue").isInstance(o);
  }

  public static boolean isAMemberOf(String username, String userlist) throws Exception {
    String compuser;
    if (userlist == null) return false;
    StringTokenizer st = new StringTokenizer(userlist, ",", false);
    while (st.hasMoreTokens()) {
       compuser = st.nextToken();
       compuser.trim();
       if (username.equals(compuser)) return true;
    }
    return false;
  }

}
